/**
 * H801 Controller 2.0
 * 
 * @author Matthias Bosch
 * @date 05.04.2017
 * 
 * changelog:
 * 14.01.2018:
 * - "WiFi.hostname(host_name);" ergänzt, damit im Netzwerk der richtige Hostname angezeigt wird.
 * 28.05.2018:
 * - Adresse an die AliveSignal gesendet werden kann unabhängig konfiguriert werden.
 * 20.01.2019 (V1.4):
 * - Am GPIO3 (RX) kann ein DHT22 angeschlossen werden. Bei lesen kann es vorkommen, dass die angeschlossenen
 *   LEDs kurz flackern. Habe das auch mit den GPIOs 2 und 0 ausprobiert. Keine Änderung.
 *   Die DHT22 können an den 3,3 V Anschluss angeschlossen werden. Das Flackern geht auch nicht weg,
 *   wenn die 5 V genommen werden.
 *   Daher habe ich das Intervall zum auslsen auf 10 Minuten eingestellt.
 * 27.01.2019 (v1.5):
 * ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG ACHTUNG 
 * - Damait das Flackern weg geht muss in der DHT Libary in der Datei "DHT.cpp" (Version 1.3.1) in der Routine "DHT::read(bool force)" die Zeile 178 "InterruptLock lock;" auskommentiert werden.
 */

#include <FS.h>                   //this needs to be first, or it all crashes and burns...

#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino

//needed for library
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager
#include <ArduinoOTA.h>
#include <DHT.h>

#include <ArduinoJson.h>          //https://github.com/bblanchon/ArduinoJson

ESP8266WebServer server(80);      // Server auf Port 80 definieren
WiFiUDP Udp;

/*Parameter, die im Filesystem persitiert werden.
  Wenn weitere Daten hinzukommen sollen, müssen diese hier und
  in den Routinen "load_config" und "save_config" eingetragen werden.
*/
char          host_name[80] = "noname";             // Name des Devices im Netzwerk
char          udp_send_port[6] = "22222";           // Port an den die Daten gesendet werden sollen
int           udp_send_port_i  = 22222;             // Port an den die Daten gesendet werden als Int
char          send_data_to[15] = "192.168.2.3";     // IP Adresse des empfangenden UDP Servers als Text
IPAddress     IP_UDP_Client(192,168,2,3);           // IP Adresse des empfangenden UDP Servers als IP-Adresse

// Daten für AliveSignal
String        aliveConfigFile = "/aliveConfig.json"; // Datei in der die Konfiguartion gespeichert ist
IPAddress     aliveIp(192,168,2,40);           // IP Adresse an die das AliveSignal gesendet wird
int           alivePort = 8802;                      // Port an die das AliveSignal gesendet wird
int           aliveIntervall = 10000;                // Intervall (ms) in dem das Alive Signal gesendet werden soll
unsigned long nextCheckAlive;                      // enthält die Systemzeit zu der das nächte mal das Alive Signal gesendet werden soll

//flag for saving data
bool          shouldSaveConfig = false;

// DHT22 Config
#define DHTPIN 3                                    // GPIO 3; what digital pin we're connected to
#define DHTTYPE DHT22                               // DHT 22
DHT dht(DHTPIN, DHTTYPE);                           // Initialize DHT sensor.
float         temperatur = 0;                       // enthält den letzten gültigen Messwert für Temperatur
float         humidity = 0;                         // enthält den letzten gültigen Messwert für die Luftfeuchtigkeit
unsigned long next_check = 0;                       // enthält die Systemzeit zu der das nächte mal die Temperatur/ Luftfeutchtigkeit gelesen werden soll
// ACHTUNG: Wenn es dazu kommen, das die angeschlossenen LEDs flackern, wenn der DHT ausgelesen wird,
//          dann siehe notwendige Änderung in der DHT Lib (siehe Doku oben der Version 1.5.
int           intervall = 10000;                    // Intervall in dem die Temperatur und Luftfeuchtigkeit gemessen wird in ms

// RGB PINs
#define redPIN    15
#define greenPIN  13
#define bluePIN   12
// W1, W2 Pins
#define w1PIN     14
#define w2PIN     4

// onbaord green LED D1
#define LEDPIN    1      // für reale Schaltung Pin 1 verwenden
// onbaord red LED D2
#define LED2PIN   5      // GPIO 5

#define LEDoff digitalWrite(LEDPIN,HIGH)
#define LEDon digitalWrite(LEDPIN,LOW)

#define LED2off digitalWrite(LED2PIN,HIGH)
#define LED2on digitalWrite(LED2PIN,LOW)

int led_delay_red             = 0;
int led_delay_green           = 0;
int led_delay_blue            = 0;
int led_delay_w1              = 0;
int led_delay_w2              = 0;

char          t_slide_char[6] = "1";      // Port an den die Daten gesendet werden sollen
int           t_slide         = 1;        // Dämpfung mit der Farbwertänderungen durchgeführt werden.
long          m_zeit          = 0;        // Systemzeit an der die letzte Änderung der Farbwerte durchgeführt wurde oder an der ein neuer Farbwert gesetzt wurde
int           RED, GREEN, BLUE, W1, W2;   // Zielfarbwerte
int           RED_A = 0;                  // aktueller Farbwert
int           GREEN_A = 0;                // aktueller Farbwert
int           BLUE_A = 0;                 // aktueller Farbwert
int           W1_A = 0;                   // aktueller Farbwert
int           W2_A = 0;                   // aktueller Farbwert

void setup() {
  // put your setup code here, to run once:
  // Serial.begin(115200);
  // Serial.println();

  // wenn der ESP8266 noch andere Parameteradaten gespeichert hat, kann es vorkommen,
  // dass er nach dem flashen abstürzt. Dann einmal das Filesystem neu flashen = formatfs();
  // auskommentieren und danach wieder rein nehmen. Dann müssen die Daten neu gesepeichert werden.
  // formatfs();

  //load configuration from FS json
  load_config();

  // laden der Daten AliveSignal
  loadAliveConfig(aliveIp, alivePort, aliveIntervall, aliveConfigFile);

  WiFi.hostname(host_name);

  //WiFiManager
  //Local intialization. Once its business is done, there is no need to keep it around
  WiFiManager wifiManager;

  //set config save notify callback
  wifiManager.setSaveConfigCallback(saveConfigCallback);

  //set static ip
  // wifiManager.setSTAStaticIPConfig(IPAddress(10,0,1,99), IPAddress(10,0,1,1), IPAddress(255,255,255,0));

  //set minimu quality of signal so it ignores AP's under that quality
  //defaults to 8%
  //wifiManager.setMinimumSignalQuality();
  
  //sets timeout until configuration portal gets turned off
  //useful to make it all retry or go to sleep
  //in seconds
  wifiManager.setTimeout(120);

  //fetches ssid and pass and tries to connect
  //if it does not connect it starts an access point with the specified name
  //here  "AutoConnectAP"
  //and goes into a blocking loop awaiting configuration
  if (!wifiManager.autoConnect(host_name, "password")) {
    Serial.println("failed to connect and hit timeout");
    delay(3000);
    //reset and try again, or maybe put it to deep sleep
    ESP.reset();
    delay(5000);
  }

  //if you get here you have connected to the WiFi
  Serial.println("connected...yeey :)");

  Serial.println("local ip");
  Serial.println(WiFi.localIP());

  // Start OTA server.
  ArduinoOTA.setHostname(host_name);
//  ArduinoOTA.setHostname((const char *)host_name.c_str());
  ArduinoOTA.begin();

// melden, per UDP, dass ein Neustart erfolgt ist
   String tempString;
   tempString = "Reboot=";
   tempString += host_name;
   tempString += "; Reason=";
   tempString += ESP.getResetReason();
   tempString += "; ReasonID=";
   rst_info* rinfo = ESP.getResetInfoPtr();
   tempString += rinfo->reason;
   sendUDP(tempString);

// --------------------------------------------------------------
// --------- Code für Web-Server --------------------------------
// --------------------------------------------------------------
  server.on ( "/reset", reset );
  server.on ( "/reboot", reboot );
  server.on ( "/formatfs", formatfs );
  server.on ( "/readconfig", read_config );
  server.on ( "/saveconfig", save_config );
  server.on ( "/alivesaveconfig", httpSaveAliveConfig );
  server.on ( "/help", help );
  server.on ( "/", help );
  server.on ( "/set", set );

// --- sketchspezifische URIs ---
  server.on ( "/temperatur", httptemperatur );

  server.onNotFound ( handleNotFound );
  server.begin();


// --------------------------------------------------------------
// --------- Konfiguration der Angeschlossenen Geräte -----------
// --------------------------------------------------------------
    
  pinMode(LEDPIN, OUTPUT);  
  pinMode(LED2PIN, OUTPUT);  
  
  pinMode(redPIN, OUTPUT);
  pinMode(greenPIN, OUTPUT);
  pinMode(bluePIN, OUTPUT);
  pinMode(w1PIN, OUTPUT);
  pinMode(w2PIN, OUTPUT);    

  analogWriteRange(100);

// --------------------------------------------------------------
// --------- Für DHT22 ------------------------------------------
// --------------------------------------------------------------

  dht.begin();

}

void loop() {

  // Handle OTA server.
  ArduinoOTA.handle();

  server.handleClient();
  yield();
  handleAliveSignal();
  handleDHT();

  // die Farbwerte werden langsam eingestellt
  if (millis() > m_zeit + t_slide) {            // wenn die Zeit seit der letzten Änderung vergangen ist...
    LED_RED();
    delay(1);
    LED_GREEN();
    delay(1);
    LED_BLUE();
    delay(1);
    LED_W1();
    delay(1);
    LED_W2();
    m_zeit = millis();                          // jetzt merken wir uns wieder die aktuelle Zeit
  }

}

/*----------------------------------------------------------------------
  ------------- Ab hier kommen Hilfsroutinen aus dem Template ----------
  ----------------------------------------------------------------------*/
void reset() {
  String response = "";
  response = "Die WLAN Zugangsdaten werden geloescht und das Geraet neu gestartet.";
  response += "Nach dem Neustart oeffnet das Geraet einen Access-Point ueber den die Konfiguration";
  response += "neu eingestellt werden kann.";
  server.send ( 200, "text/html", response);
  Serial.println("Reset to Default and Reboot...");
  sendUDP("Reset WLAN Accessdata=1");

  delay(1000);

  WiFiManager wifiManager;
  //reset settings
  wifiManager.resetSettings();

  delay(1000);
  ESP.reset();
}

void reboot() {
  String response = "";
  response = "Softwarerestart erfolgt";
  server.send ( 200, "text/html", response);
  sendUDP("Softwarereboot initiated=1");
  delay(1000);
  ESP.restart();
}

void formatfs() {
  String response = "";
  response = "Filesystem formated.<br>";
  response += "Confing deleted.";
  server.send ( 200, "text/html", response);
  Serial.println("Filesystem formated. Config deleted...");

  delay(1000);

  //clean FS
  SPIFFS.format();

  delay(1000);
}

void handleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";

  for ( uint8_t i = 0; i < server.args(); i++ ) {
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
  }

  server.send ( 404, "text/plain", message );
}

//callback notifying us of the need to save config
void saveConfigCallback () {
  Serial.println("Should save config");
  shouldSaveConfig = true;
}

// Routine zum senden des übergebenen Textes per UDP
void sendUDP(String Text) {
   int tempLen = Text.length()+1;
   char tempChar[tempLen];
   Text.toCharArray(tempChar,tempLen);
   Udp.beginPacket(IP_UDP_Client, udp_send_port_i);
   Udp.write(tempChar);
   Udp.endPacket();
}

// Routine zum senden des übergebenen Textes per UDP
void sendUDP(String Text, IPAddress ip, int port) {
   int tempLen = Text.length()+1;
   char tempChar[tempLen];
   Text.toCharArray(tempChar,tempLen);
   Udp.beginPacket(ip, port);
   Udp.write(tempChar);
   Udp.endPacket();
}

// sendet bei regelmäßigem Aufruf ein Alive Signal an die konfigurierte Adresse und per UDP
void handleAliveSignal() {
  // prüfen ob das Alive Signal gesendet werden sollen;
  if (millis() > nextCheckAlive) {
    sendUDP("online=1", aliveIp, alivePort);
    nextCheckAlive = millis() + aliveIntervall;
  }
}

