// Laden der Konfigurationsdaten aus dem Speicher
void load_config() {
  //load configuration from FS json
  Serial.println("mounting FS...");

  if (SPIFFS.begin()) {
    Serial.println("mounted file system");
    if (SPIFFS.exists("/config.json")) {
      //file exists, reading and loading
      Serial.println("reading config file");
      File configFile = SPIFFS.open("/config.json", "r");
      if (configFile) {
        Serial.println("opened config file");
        size_t size = configFile.size();
        // Allocate a buffer to store contents of the file.
        std::unique_ptr<char[]> buf(new char[size]);

        configFile.readBytes(buf.get(), size);
        DynamicJsonBuffer jsonBuffer;
        JsonObject& json = jsonBuffer.parseObject(buf.get());
        json.printTo(Serial);
        if (json.success()) {
          Serial.println("\nparsed json");

          strcpy(host_name, json["host_name"]);
          strcpy(send_data_to, json["send_data_to"]);
          strcpy(udp_send_port, json["udp_send_port"]);
          strcpy(t_slide_char, json["t_slide"]);

        } else {
          Serial.println("failed to load json config");
        }
      }
    }
  } else {
    Serial.println("failed to mount FS");
  }
  //end loading

  // Wandeln des IP Stings in die IP-Adresse
  if (IP_UDP_Client.fromString(send_data_to)) {
    // it was a valid address, do something with it 
    Serial.println("IP-Adresse erfolgreich gewandelt");
  }

  // Wandeln des Port Char in einen Integer Wert
  String tempStr(udp_send_port);
  if (tempStr.toInt() != 0) {
    udp_send_port_i = tempStr.toInt();
    Serial.print("UDP Port erfolgreich gewandelt in ");
    Serial.println(tempStr);
  }

  // Wandeln des t_slide Char in einen Integer Wert
  String tempStr2(t_slide_char);
  if (tempStr2.toInt() != 0) {
    t_slide = tempStr2.toInt();
    Serial.print("Daempfung erfolgreich gewandelt in ");
    Serial.println(tempStr2);
  }
}

// Funktion um einen String mit einer IP-Adresse in eine IP-Adresse zu wandeln
IPAddress TextToIP(String IP_in_Text) {
  IPAddress ipa;
  // Wandeln des Stings in die IP-Adresse
  if (ipa.fromString(IP_in_Text)) {
    // it was a valid address, do something with it 
    Serial.println("IP-Adresse erfolgreich gewandelt");
    return ipa;
  }  
}

// Lesen der Konfiguration auf Anfrage per HTTP und Rückgabe der Werte
void read_config() {
  String response = "";

  response = "Hostname = ";
  response += host_name;
  response += "<br>";
  response += "Zieladresse = ";
  response += send_data_to;
  response += "<br>";
  response += "Zielport = ";
  response += udp_send_port;
  response += "<br>";
  response += "Daempfung = ";
  response += t_slide;
  server.send ( 200, "text/html", response);
}

// Speichern per HTTP übergebener Werte
void save_config() {
  String response = "";
  String temp = "";

  /* ------------------------------------------------------------------------------*/
  // prüfen auf "hostname"
  if (server.hasArg("hostname")) {
      // hostname auslesen
      temp = server.arg("hostname");
      response += "neuer Hostname = ";
      response += temp;
      response += "<br>";
      Serial.print("neuer Hostname = ");
      Serial.println(temp);
      temp.toCharArray(host_name,80);    // Speichern in der Char Variablen
  }

  /* ------------------------------------------------------------------------------*/
  // prüfen auf "senddatato"
  if (server.hasArg("senddatato")) {
      // senddatato auslesen
      temp = server.arg("senddatato");
      response += "neue Zieladresse = ";
      response += temp;
      response += "<br>";
      Serial.print("neue Zieladresse = ");
      Serial.println(temp);
      temp.toCharArray(send_data_to,15);    // Speichern in der Char Variablen
      // Wandeln des Stings in die IP-Adresse
      if (IP_UDP_Client.fromString(temp)) {
        // it was a valid address, do something with it 
        Serial.println("IP-Adresse erfolgreich gewandelt");
      }
  }

  /* ------------------------------------------------------------------------------*/
  // prüfen auf "udpport"
  if (server.hasArg("udpport")) {
      // udpport auslesen
      temp = server.arg("udpport");
      response += "neuer Zielport = ";
      response += temp;
      response += "<br>";
      Serial.print("neuer Zielport = ");
      Serial.println(temp);
      temp.toCharArray(udp_send_port,6);    // Speichern in der Char Variablen
  }

  /* ------------------------------------------------------------------------------*/
  // prüfen auf "t_slide"
  if (server.hasArg("t_slide")) {
      // t_slide auslesen
      temp = server.arg("t_slide");
      response += "neue Daempfung = ";
      response += temp;
      response += "<br>";
      Serial.print("neue Daempfung = ");
      Serial.println(temp);
      temp.toCharArray(t_slide_char,6);    // Speichern in der Char Variablen
      // Wandeln des t_slide Char in einen Integer Wert
      String tempStr2(t_slide_char);
      if (tempStr2.toInt() != 0) {
        t_slide = tempStr2.toInt();
        Serial.print("Daempfung erfolgreich gewandelt in ");
        Serial.println(tempStr2);
      }
  }


  //save the custom parameters to FS
  Serial.println("saving config");
  DynamicJsonBuffer jsonBuffer;
  JsonObject& json = jsonBuffer.createObject();
  json["host_name"] = host_name;
  json["send_data_to"] = send_data_to;
  json["udp_send_port"] = udp_send_port;
  json["t_slide"] = t_slide_char;

  File configFile = SPIFFS.open("/config.json", "w");
  if (!configFile) {
    Serial.println("failed to open config file for writing");
  }

  json.printTo(Serial);
  json.printTo(configFile);
  configFile.close();
  //end save

  response += "Saved Configuration<br>";
  server.send ( 200, "text/html", response);

}

